"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var test_utils_1 = require("@vue/test-utils");
var proppy_1 = require("proppy");
var attach_1 = require("./attach");
describe('proppy-vue :: attach', function () {
    test('is a function', function () {
        expect(typeof attach_1.attach).toEqual('function');
    });
    test('single render', function () {
        var Base = {
            name: 'Base',
            props: ['counter'],
            render: function (h) {
                return h('span', null, "Counter: " + this.counter);
            },
        };
        var P = proppy_1.withProps({
            counter: 1,
        });
        var Enhanced = attach_1.attach(P)(Base);
        var Root = {
            name: 'Root',
            provide: {
                proppy: {},
            },
            render: function (h) {
                return h(Enhanced, null);
            },
        };
        var wrapper = test_utils_1.mount(Root);
        expect(wrapper.find('span').text()).toEqual('Counter: 1');
    });
    test('single render with parent props', function () {
        var Base = {
            name: 'Base',
            props: ['counter', 'foo'],
            render: function (h) {
                return h('span', null, "Counter: " + this.counter + ", Foo: " + this.foo);
            },
        };
        var P = proppy_1.withProps({
            counter: 1,
        });
        var Enhanced = attach_1.attach(P)(Base);
        var Root = {
            name: 'Root',
            provide: {
                proppy: {},
            },
            render: function (h) {
                return h(Enhanced, {
                    props: {
                        foo: 'foo value',
                    },
                });
            },
        };
        var wrapper = test_utils_1.mount(Root);
        expect(wrapper.find('span').text()).toEqual('Counter: 1, Foo: foo value');
    });
    test('single render while accessing providers', function () {
        var Base = {
            name: 'Base',
            props: ['counter', 'bar'],
            render: function (h) {
                return h('span', null, "Counter: " + this.counter + ", Bar: " + this.bar);
            },
        };
        var P = proppy_1.withProps(function (props, providers) {
            return {
                counter: 1,
                bar: providers.bar,
            };
        });
        var Enhanced = attach_1.attach(P)(Base);
        var Root = {
            name: 'Root',
            provide: {
                proppy: {
                    bar: 'bar value',
                },
            },
            render: function (h) {
                return h(Enhanced, null);
            },
        };
        var wrapper = test_utils_1.mount(Root);
        expect(wrapper.find('span').text()).toEqual('Counter: 1, Bar: bar value');
    });
    test('re-renders based on own updates', function () {
        var Base = {
            name: 'Base',
            props: ['counter', 'setCounter'],
            render: function (h) {
                var _this = this;
                return h('div', null, [h('p', null, ['Counter: ', this.counter]), h('button', {
                        on: {
                            click: function () {
                                return _this.setCounter(_this.counter + 1);
                            }
                        }
                    }, ['Increment'])]);
            },
        };
        var P = proppy_1.withState('counter', 'setCounter', 0);
        var Enhanced = attach_1.attach(P)(Base);
        var Root = {
            name: 'Root',
            provide: {
                proppy: {},
            },
            render: function (h) {
                return h(Enhanced, null);
            },
        };
        var wrapper = test_utils_1.mount(Root);
        expect(wrapper.find('p').text()).toEqual('Counter: 0');
        wrapper.find('button').trigger('click');
        expect(wrapper.find('p').text()).toEqual('Counter: 1');
    });
    test('re-renders based on parent updates', function () {
        var Base = {
            name: 'Base',
            props: ['counter'],
            render: function (h) {
                return h('p', null, ['Counter: ', this.counter]);
            },
        };
        var P = proppy_1.withState('counter', 'setCounter', 0);
        var Enhanced = attach_1.attach(P)(Base);
        var Root = {
            name: 'Root',
            provide: {
                proppy: {},
            },
            data: function () {
                return {
                    counter: 0,
                };
            },
            methods: {
                setCounter: function (n) {
                    this.counter = n;
                },
            },
            render: function (h) {
                var _this = this;
                return h('div', null, [h('button', {
                        on: {
                            click: function () {
                                return _this.setCounter(_this.counter + 1);
                            }
                        }
                    }, ['Increment']), h(Enhanced, {
                        attrs: { counter: this.counter }
                    }, [])]);
            },
        };
        var wrapper = test_utils_1.mount(Root);
        expect(wrapper.find('p').text()).toEqual('Counter: 0');
        wrapper.find('button').trigger('click');
        expect(wrapper.find('p').text()).toEqual('Counter: 1');
    });
});
//# sourceMappingURL=attach.spec.js.map